from xl.reader.inject import inject_imports
from xl.reader.excel_reader import SynoExcelReader
from xl.reader.style import parse_font, parse_fill, parse_alignment, parse_border, get_color, parse_dxf_fill_bg
from xl.reader.json_io import JSONWriter, export_json
from xl.reader.utils import boundaries_to_index, random_string, get_xml_value
from xl.common.constant import (
    DEFAULT_ROW_COUNT,
    DEFAULT_COL_COUNT,
    MIN_COL_WIDTH,
    MAX_COL_WIDTH,
    MIN_ROW_HEIGHT,
    MAX_ROW_HEIGHT,
    ROW_HEIGHT_RATIO,
    COL_WIDTH_RATIO,
    ROW_HEIGHT,
    COL_WIDTH,
    ROW_HEADER_WIDTH,
    COL_HEADER_HEIGHT,
    SCHEMA_VERSION,
)
from native.json import rewrite_cells

from openpyxl.utils.indexed_list import IndexedList
from openpyxl.utils.cell import range_boundaries, SHEETRANGE_RE, range_to_tuple
from openpyxl.styles.numbers import BUILTIN_FORMATS
from openpyxl.worksheet.dimensions import (
    ColumnDimension,
    RowDimension,
)
from openpyxl.utils.units import EMU_to_pixels
from openpyxl.drawing.spreadsheet_drawing import (
    TwoCellAnchor,
    OneCellAnchor,
    AbsoluteAnchor,
)
from operator import itemgetter
import os
import sys


def write_cells(ws, fp, remaining_quota):
    max_row = 0
    max_col = 0
    cells_count = 0

    fp.write('"cells":{')
    row_count = 0
    for row in ws.rows:
        col_count = 0
        for cell in row:
            row = cell.row
            col = cell.column
            value = cell.value
            xf_id = cell.style_id

            if row > max_row:
                max_row = row
            if col > max_col:
                max_col = col

            has_comment = False
            cell_config = {}
            if xf_id > 0:
                cell_config['s'] = xf_id
            if value is not None:
                cell_config['v'] = value
            if row in ws.comments and col in ws.comments[row]:
                cell_config['n'] = ws.comments[row][col]
                has_comment = True

            # strip empty cell
            if value is None and not has_comment and (row >= 1000 or col >= 40):
                continue

            cells_count += 1
            if cells_count > remaining_quota:
                sys.exit(255)

            if col_count == 0:
                if row_count > 0:
                    # row separator
                    fp.write(',')
                # start row
                fp.write('"%d":{' % row)
            else:
                # col separator
                fp.write(',')
            fp.write('"%d":' % col)
            fp.write_json(cell_config)
            col_count += 1
        # this row is not empty
        if col_count > 0:
            fp.write('}')
            row_count += 1
    fp.write('},')
    return (max_row, max_col, cells_count)


def get_dimensions(ws, max_col_index):
    global_xf = 0
    max_col = max_col_index + 1

    col_infos = {}
    for _, data in ws.column_dimensions.items():
        col_info = {}
        style_id = None
        if 'style' in data:
            style_id = int(data['style'])
            data['style'] = ws.parent._cell_styles[style_id]
            if style_id > 0:
                col_info['s'] = style_id
        cd = ColumnDimension(ws, **data)
        # Office#5042
        if cd.min > max_col:
            continue
        if cd.max > max_col and cd.max != 16384:
            cd.max = max_col

        if cd.customWidth:
            width = min(max(cd.width / COL_WIDTH_RATIO,
                            MIN_COL_WIDTH), MAX_COL_WIDTH)
            col_info['w'] = int(round(width))
        if cd.hidden:
            col_info['hidden'] = True
        if not col_info:
            continue
        if cd.max == 16384 and style_id:
            global_xf = style_id
            cd.max = max_col
        for index in range(cd.min - 1, cd.max):
            col_infos[index] = col_info.copy()

    row_infos = {}
    for _, data in ws.row_dimensions.items():
        row_info = {}
        if 's' in data:
            style_id = int(data['s'])
            data['s'] = ws.parent._cell_styles[style_id]
            if style_id > 0:
                row_info['s'] = style_id
        rd = RowDimension(ws, **data)
        if rd.customHeight:
            height = min(max(rd.ht / ROW_HEIGHT_RATIO,
                             MIN_ROW_HEIGHT), MAX_ROW_HEIGHT)
            row_info['h'] = int(round(height))
        if rd.hidden:
            row_info['hidden'] = True
        if not row_info:
            continue
        row_infos[rd.index - 1] = row_info

    return {'rows': row_infos, 'cols': col_infos, 'global': {'s': global_xf} if global_xf > 0 else None}


def get_merged_cells(ws):
    merged_cells = []
    if not ws.merged_cells:
        return merged_cells
    for cr in ws.merged_cells.mergeCell:
        merged_cells.append(boundaries_to_index(cr.bounds))
    return merged_cells


def get_view(ws):
    fixed_row = 0
    fixed_col = 0
    if ws.views.sheetView and ws.views.sheetView[0].pane:
        pane = ws.views.sheetView[0].pane
        if pane.state in ('frozen', 'frozenSplit'):
            if pane.xSplit:
                fixed_row = int(pane.xSplit)
            if pane.ySplit:
                fixed_col = int(pane.ySplit)
    return {'fixedRowTop': fixed_row, 'fixedColumnLeft': fixed_col}


def get_ws_properties(ws):
    snapshot = {}
    if ws.sheet_properties and ws.sheet_properties.tabColor:
        tab_color = get_color(ws.sheet_properties.tabColor, ws.parent)
        if tab_color:
            snapshot['tabColor'] = tab_color
    if ws.sheet_state != 'visible':
        snapshot['hidden'] = True
    return snapshot


def get_cfs(ws):
    cfs = []
    wb = ws.parent
    supported_types = set([
        'cellIs', 'containsBlanks', 'containsError',
        'containsText', 'beginsWith', 'endsWith',
        'notContainsBlanks', 'notContainsText', 'expression'
    ])
    for cf in ws.formatting:
        for rule in cf.rules:
            if rule.type not in supported_types:
                continue
            dxf = None
            if rule.dxfId is not None:
                dxf = wb._differential_styles[rule.dxfId]
            cf_config = {
                'cfId': random_string(),
                'type': rule.type,
            }
            if rule.operator:
                cf_config['op'] = rule.operator
            if rule.text:
                cf_config['text'] = rule.text
            if rule.formula:
                cf_config['f1'] = get_xml_value(rule.formula[0])
                if len(rule.formula) > 1:
                    cf_config['f2'] = get_xml_value(rule.formula[1])
            cf_config['ranges'] = [str(r) for r in cf.sqref.ranges]
            style = cf_config['style'] = {}
            if dxf and dxf.font:
                font = dxf.font
                if font.b:
                    style['bold'] = True
                if font.i:
                    style['italic'] = True
                if font.strike:
                    style['strike'] = True
                if font.u:
                    style['underline'] = True
                if font.color:
                    font_color = get_color(font.color, wb)
                    if font_color:
                        style['color'] = font_color
            if dxf and dxf.fill:
                bg_color = parse_dxf_fill_bg(dxf.fill, wb)
                if bg_color:
                    style['bgColor'] = bg_color

            cfs.append((rule.priority, cf_config))
    return list(map(itemgetter(1), sorted(cfs, key=itemgetter(0))))


def get_filter(ws):
    if not ws.auto_filter:
        return None
    filter_config = {
        'range': boundaries_to_index(range_boundaries(ws.auto_filter.ref)),
        'enabled': True,
        'id': random_string(),
        'filter': {}
    }
    for column in ws.auto_filter.filterColumn:
        cond = None
        if column.customFilters:
            cond = {'type': 'custom', 'value': [], 'operator': [],
                    'and': bool(column.customFilters._and)}
            for custom_filter in column.customFilters.customFilter:
                operator = custom_filter.operator
                value = get_xml_value(custom_filter.val)
                if operator is not None and value is not None:
                    cond['value'].append(value)
                    cond['operator'].append(operator)

            if len(cond['value']) == 1 and isinstance(cond['value'][0], (str, bytes)):
                operator = cond['operator'][0]
                value = cond['value'][0]
                headWildcard = value[0] == '*'
                tailWildcard = value[-1] == '*'
                if operator == 'equal':
                    if len(value) > 1 and headWildcard and tailWildcard:
                        operator = 'containsTexts'
                        value = value[1:-1]
                    elif len(value) > 1 and headWildcard and not tailWildcard:
                        operator = 'endsWith'
                        value = value[1:]
                    elif len(value) > 1 and not headWildcard and tailWildcard:
                        operator = 'beginsWith'
                        value = value[:-1]
                    else:
                        operator = 'exactTexts'
                    cond['type'] = 'text'
                elif operator == 'notEqual':
                    if len(value) > 1 and headWildcard and tailWildcard:
                        operator = 'notContainsTexts'
                        value = value[1:-1]
                    cond['type'] = 'text'
                cond['operator'][0] = operator
                cond['value'][0] = value
        elif column.filters:
            cond = {'type': 'value', 'value': []}
            if column.filters.blank:
                cond['value'].append('')
            for filter_rule in column.filters.filter:
                cond['value'].append(filter_rule)

        if cond:
            filter_config['filter'][column.colId - 1] = cond

        if filter_config['filter']:
            max_key = max(filter_config['filter'].keys())
            for index in range(0, max_key):
                if index not in filter_config['filter']:
                    filter_config['filter'][index] = None

    return filter_config


def get_validations(ws):
    validations = {}
    if not ws.data_validations:
        return validations

    def get_dv_msg(title, content):
        msg = ''
        if title:
            msg += title
        if content:
            msg += ' - ' if msg else '' + content
        return msg

    for validation in ws.data_validations.dataValidation:
        dv_config = {}
        dv_type = validation.type
        if dv_type == 'custom':
            dv_config['type'] = 'custom'
            dv_config['operator'] = 'custom'
        elif dv_type == 'date':
            dv_config['type'] = 'date'
        elif dv_type in ('decimal', 'whole'):
            dv_config['type'] = 'decimal'
        elif dv_type == 'list':
            dv_config['type'] = 'list'

        if 'type' not in dv_config:
            continue

        if validation.operator:
            dv_config['operator'] = validation.operator

        dv_config['reject'] = bool(
            validation.showErrorMessage and validation.errorStyle == 'stop')

        if validation.promptTitle or validation.prompt:
            dv_config['message'] = get_dv_msg(
                validation.promptTitle, validation.prompt)
        elif validation.errorTitle or validation.error:
            dv_config['message'] = get_dv_msg(
                validation.errorTitle, validation.error)
        else:
            dv_config['message'] = ''

        f1 = validation.formula1 or ''
        if dv_config['type'] == 'list':
            if len(f1) > 2 and f1[0] == '"' and f1[-1] == '"':
                dv_config['operator'] = 'list'
                f1 = f1[1:-1]
            else:
                dv_config['operator'] = 'listRange'
                if f1 in ws.parent.defined_names:
                    f1 = ws.parent.defined_names[f1].value
        elif dv_config['type'] in ('decimal', 'date'):
            f1 = get_xml_value(f1)
        dv_config['f1'] = f1

        if validation.formula2:
            f2 = validation.formula2 or ''
            if dv_config['type'] in ('decimal', 'date'):
                f2 = get_xml_value(f2)
            dv_config['f2'] = f2

        dv_config['ranges'] = [str(r) for r in validation.sqref.ranges]
        dv_config['validId'] = random_string()
        validations[dv_config['validId']] = dv_config

    return validations


def convert_chart(chart):

    def rich_to_plain(rich):
        plain = ''
        for paragraph in rich.paragraphs:
            for text_run in paragraph.text:
                plain += text_run.value
        return plain

    def convert_chart_title(title):
        text = ''
        if title.text is not None:
            if title.text.strRef is not None and title.text.strRef.f is not None:
                text = title.text.strRef.f
            if title.text.rich is not None:
                text = rich_to_plain(title.text.rich)
        return {
            'text': text
        }

    # Office bug #5005
    def trim_reference(reference):
        ref = reference.strip()
        if ref[0] == '(' and ref[-1] == ')':
            ref = ref[1:-1]
        return ref

    supported_chart_type = ['bar', 'line', 'area', 'pie', 'scatter']

    chart_type = 'bar'
    bar_dir = 'col'
    grouping = 'standard'
    series = []
    x_axis = None
    y_axis = None
    legend = None
    plot_area = {}
    chart_type_collection = set()

    for chart_instance in chart._charts:
        if chart_instance._series_type not in supported_chart_type:
            continue
        chart_type = chart_instance._series_type
        chart_type_collection.add(chart_type)
        if chart_type == 'bar':
            bar_dir = chart_instance.barDir
        if chart_type in ('bar', 'line', 'area'):
            grouping = chart_instance.grouping
        if chart_type != 'pie':
            if x_axis is None and chart_instance.x_axis is not None:
                x_axis = chart_instance.x_axis
            if y_axis is None and chart_instance.y_axis is not None:
                y_axis = chart_instance.y_axis
        if legend is None and chart_instance.legend is not None:
            legend = chart_instance.legend

        for ser in chart_instance.ser:
            ser_object = {
                'type': chart_type
            }
            val = ser.val if chart_type != 'scatter' else ser.yVal
            if val is not None and val.numRef is not None and val.numRef.f is not None:
                ser_object['val'] = trim_reference(val.numRef.f)
            else:
                continue
            if ser.tx is not None and ser.tx.strRef is not None and ser.tx.strRef.f is not None:
                ser_object['tx'] = trim_reference(ser.tx.strRef.f)
            cat = ser.cat if chart_type != 'scatter' else ser.xVal
            cat_ref = None if cat is None else cat.numRef if cat.strRef is None else cat.strRef
            if cat_ref is not None and cat_ref.f is not None:
                ser_object['cat'] = trim_reference(cat_ref.f)

            if chart_type in ('line', 'scatter') and ser.marker is not None:
                ser_object['marker'] = {
                    'symbol': ser.marker.symbol if ser.marker.symbol is not None else 'none',
                    'size': ser.marker.size if ser.marker.size is not None else 5
                }

            if chart_type == 'line' and ser.smooth:
                ser_object['smooth'] = True

            series.append(ser_object)

    # pylint: disable=len-as-condition
    if len(chart_type_collection) > 2 or len(chart_type_collection) < 1 or len(series) < 1:
        return None
    if len(chart_type_collection) == 2:
        if 'bar' not in chart_type_collection or 'line' not in chart_type_collection:
            return None
        chart_type = 'hybrid'

    plot_area['series'] = series
    if x_axis is not None and x_axis.title is not None:
        plot_area['xAxis'] = {
            'title': convert_chart_title(x_axis.title)
        }
    if y_axis is not None and y_axis.title is not None:
        plot_area['yAxis'] = {
            'title': convert_chart_title(y_axis.title)
        }

    chart_config = {
        'plotArea': plot_area,
        'chartType': chart_type,
        'barDir': bar_dir,
        'grouping': grouping
    }

    if chart.title is not None:
        chart_config['title'] = convert_chart_title(chart.title)
    if legend is not None:
        chart_config['legend'] = {
            'position': legend.legendPos
        }
    return chart_config


def get_drawings(reader, ws, dims, image_dirs):
    drawings = []

    rows = dims['rows']
    cols = dims['cols']

    def get_col_to_x(col, offset):
        x = 0
        for index in range(0, col):
            if index not in cols:
                x += COL_WIDTH
                continue
            col_info = cols[index]
            if not col_info.get('hidden', False):
                x += col_info.get('w', COL_WIDTH)
        return x + EMU_to_pixels(offset)

    def get_row_to_y(row, offset):
        y = 0
        for index in range(0, row):
            if index not in rows:
                y += ROW_HEIGHT
                continue
            row_info = rows[index]
            if not row_info.get('hidden', False):
                y += row_info.get('h', ROW_HEIGHT)
        return y + EMU_to_pixels(offset)

    def anchor_to_pos(anchor):
        top = 0
        left = 0
        width = 0
        height = 0
        if isinstance(anchor, AbsoluteAnchor):
            top = EMU_to_pixels(anchor.pos.y)
            left = EMU_to_pixels(anchor.pos.x)
            width = EMU_to_pixels(anchor.ext.cx)
            height = EMU_to_pixels(anchor.ext.cy)
        elif isinstance(anchor, TwoCellAnchor):
            top = get_row_to_y(anchor._from.row, anchor._from.rowOff)
            left = get_col_to_x(anchor._from.col, anchor._from.colOff)
            width = get_col_to_x(anchor.to.col, anchor.to.colOff) - left
            height = get_row_to_y(anchor.to.row, anchor.to.rowOff) - top
        elif isinstance(anchor, OneCellAnchor):
            top = get_row_to_y(anchor._from.row, anchor._from.rowOff)
            left = get_col_to_x(anchor._from.col, anchor._from.colOff)
            width = EMU_to_pixels(anchor.ext.cx)
            height = EMU_to_pixels(anchor.ext.cy)
        return {
            'top': top + COL_HEADER_HEIGHT,
            'left': left + ROW_HEADER_WIDTH,
            'width': width,
            'height': height
        }

    for chart in ws.charts:
        chart_object = convert_chart(chart)
        if chart_object is None:
            continue
        chart_config = {
            'id': random_string(),
            'chartObj': chart_object,
            'type': 'chart',
            'pos': anchor_to_pos(chart.anchor)
        }
        drawings.append(chart_config)

    for image in ws.images:
        image_id = random_string()
        if copy_image(reader, image_dirs, image['name'], image_id):
            drawings.append({
                'id': image_id,
                'type': 'img',
                'pos': anchor_to_pos(image['anchor'])
            })

    return drawings


def get_stylesheet(wb):
    stylesheet = {
        'styles': [],
        'fonts': [],
        'fills': [],
        'alignments': [],
        'borders': [],
        'numberFormats': [],
    }
    # TODO
    # default_alignmentId = wb._cell_styles[0].alignmentId
    stylesheet['styles'] = [
        [0, s.fontId, s.fillId, s.alignmentId,
            s.borderId, s.numFmtId, int(s.quotePrefix)]
        for s in wb._cell_styles[1:]
    ]

    # fonts
    stylesheet['fonts'] = [parse_font(f, wb) for f in wb._fonts[1:]]
    # fills
    stylesheet['fills'] = [parse_fill(f, wb) for f in wb._fills[1:]]
    # alignments
    stylesheet['alignments'] = [parse_alignment(a) for a in wb._alignments[1:]]
    # borders
    stylesheet['borders'] = [parse_border(b, wb) for b in wb._borders[1:]]

    # number formats
    rebuilt_formats = IndexedList(['General'])
    for s in stylesheet['styles']:
        numFmtId = s[5]
        if numFmtId < 164:
            number_format = BUILTIN_FORMATS.get(numFmtId, 'General')
        else:
            number_format = wb._number_formats[numFmtId - 164]
        s[5] = rebuilt_formats.add(number_format)
    stylesheet['numberFormats'] = [{'fc': fc} for fc in rebuilt_formats[1:]]

    return stylesheet


def get_defined_names(wb):
    defined_names = {}
    for defn in wb.defined_names.definedName:
        # reject non-reference value
        if SHEETRANGE_RE.match(defn.value) is None:
            continue
        index = 'global'
        if defn.localSheetId is not None:
            index = 'sh_%d' % (defn.localSheetId + 1)
        defined_names.setdefault(index, {})[defn.name] = defn.value
    return defined_names


def get_index(wb):
    worksheets = wb.worksheets
    index = {
        'maxIndex': len(worksheets),
        'order': [],
        'sheets': {},
        'schemaVersion': SCHEMA_VERSION,
        'ver': 1,
        'gcVer': 1,
    }
    for idx, ws in enumerate(worksheets):
        sheet_id = 'sh_%d' % (idx + 1)
        index['order'].append(sheet_id)
        index['sheets'][sheet_id] = {'title': ws.title, 'deleted': False}
    return index


def setup_image_dir(output_dir):
    img_dir = os.path.join(output_dir, 'images')
    img_unzip_dir = os.path.join(output_dir, '_unzip')
    try:
        os.mkdir(img_dir, 0o770)
        os.mkdir(img_unzip_dir, 0o770)
    except OSError:
        pass
    return (img_dir, img_unzip_dir)


image_cache = {}


def copy_image(reader, image_dirs, name, image_id):
    img_dir, img_unzip_dir = image_dirs
    dest = os.path.join(img_dir, image_id)
    if name not in image_cache:
        try:
            path = reader.extract_file(name, img_unzip_dir)
        except Exception as _:
            return False
        os.rename(path, dest)
        image_cache[name] = dest
    else:
        os.link(image_cache[name], dest)
    return True


def rewrite_hyperlinks(ws, filename):
    links = []
    # TODO test quote sheetname
    for link in ws.hyperlinks.hyperlink:
        target = None
        if link.id:
            rel = ws._rels[link.id]
            target = rel.Target
        elif link.location:
            try:
                sheetname, _ = range_to_tuple(link.location)
                wb = ws.parent
                if sheetname in wb:
                    target = '#tid=%d' % (wb.index(wb[sheetname]) + 1)
            except ValueError:
                pass

        if target is None:
            continue
        min_row, min_col, max_row, max_col = boundaries_to_index(
            range_boundaries(link.ref))
        for row in range(min_row, max_row + 1):
            for col in range(min_col, max_col + 1):
                links.append((row, col, target, link.tooltip))

    if links:
        new_filename = filename + '.rewrite'
        if rewrite_cells(filename, links, new_filename):
            os.replace(new_filename, filename)


def import_xlsx(input_file, output_dir, locale):
    inject_imports(locale)

    reader = SynoExcelReader(input_file)
    reader.read()
    wb = reader.wb

    export_json(get_stylesheet(wb), os.path.join(output_dir, 'style'))
    export_json(get_defined_names(wb), os.path.join(output_dir, 'define'))
    export_json(get_index(wb), os.path.join(output_dir, 'index'))

    image_dirs = setup_image_dir(output_dir)

    remaining_quota = 2000000

    for idx, ws in enumerate(wb):
        sheet_id = 'sh_%d' % (idx + 1,)
        filename = os.path.join(output_dir, sheet_id)
        ws_props = None
        with JSONWriter(filename) as f:
            bounds = ws.get_bounds()
            max_row, max_col, cells_count = write_cells(ws, f, remaining_quota)
            remaining_quota -= cells_count
            dims = get_dimensions(ws, bounds[3] if bounds else max_col)
            f.write_dict(dims)
            f.write_key_value('rowCount', max(max_row + 1, DEFAULT_ROW_COUNT))
            f.write_key_value('colCount', max(max_col + 1, DEFAULT_COL_COUNT))
            f.write_dict(get_view(ws))
            f.write_key_value('mergeCells', get_merged_cells(ws))
            f.write_key_value('filter', get_filter(ws))
            f.write_key_value('validation', get_validations(ws))
            f.write_key_value('cfs', get_cfs(ws))
            f.write_key_value('drawing', get_drawings(
                reader, ws, dims, image_dirs), tail_comma=False)
            ws_props = get_ws_properties(ws)
        if ws_props:
            export_json(ws_props, filename + '.props')
        rewrite_hyperlinks(ws, filename)
