<?php

namespace PhpOffice\PhpPresentation\Writer\PowerPoint2007;

use PhpOffice\Common\Drawing as CommonDrawing;
use PhpOffice\Common\XMLWriter;
use PhpOffice\PhpPresentation\Style\Border;
use PhpOffice\PhpPresentation\Style\Color;
use PhpOffice\PhpPresentation\Style\Font;
use PhpOffice\PhpPresentation\Style\Fill;
use PhpOffice\PhpPresentation\Style\Outline;
use PhpOffice\PhpPresentation\Style\SchemeColor;

abstract class AbstractDecoratorWriter extends \PhpOffice\PhpPresentation\Writer\AbstractDecoratorWriter
{
    /**
     * Write relationship
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter   XML Writer
     * @param  int                            $pId         Relationship ID. rId will be prepended!
     * @param  string                         $pType       Relationship type
     * @param  string                         $pTarget     Relationship target
     * @param  string                         $pTargetMode Relationship target mode
     * @throws \Exception
     */
    protected function writeRelationship(XMLWriter $objWriter, $pId = 1, $pType = '', $pTarget = '', $pTargetMode = '')
    {
        if ($pType == '' || $pTarget == '') {
            throw new \Exception("Invalid parameters passed.");
        }
        if (strpos($pId, 'rId') === false) {
            $pId = 'rId' . $pId;
        }

        // Write relationship
        $objWriter->startElement('Relationship');
        $objWriter->writeAttribute('Id', $pId);
        $objWriter->writeAttribute('Type', $pType);
        $objWriter->writeAttribute('Target', $pTarget);

        if ($pTargetMode != '') {
            $objWriter->writeAttribute('TargetMode', $pTargetMode);
        }

        $objWriter->endElement();
    }

    /**
     * Write Border
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter    XML Writer
     * @param  \PhpOffice\PhpPresentation\Style\Border     $pBorder      Border
     * @param  string                         $pElementName Element name
     * @throws \Exception
     */
    protected function writeBorder(XMLWriter $objWriter, $pBorder, $pElementName = 'L', $lineEnd = '')
    {
        if (!($pBorder instanceof Border)) {
            return;
        }

        $lineStyle = $pBorder->getLineStyle();
        if (($lineStyle == Border::LINE_NONE || $lineStyle == Border::LINE_NOTSET) && $pElementName == '') {
            return;
        }

        $lineStyle = $pBorder->getLineStyle();
        if ($lineStyle == Border::LINE_NONE || $lineStyle == Border::LINE_NOTSET) {
            $lineStyle = Border::LINE_SINGLE;
        }
        // Line width
        $lineWidth = 12700 * $pBorder->getLineWidth();

        // a:ln $pElementName
        $objWriter->startElement('a:ln' . $pElementName);
        $objWriter->writeAttribute('w', $lineWidth);
        $objWriter->writeAttribute('cap', 'flat');
        $objWriter->writeAttribute('cmpd', $lineStyle);
        $objWriter->writeAttribute('algn', 'ctr');

        // Fill?
        if ($pBorder->getLineStyle() == Border::LINE_NONE || $pBorder->getLineStyle() == Border::LINE_NOTSET) {
            // a:noFill
            $objWriter->writeElement('a:noFill', null);
        } else {
            // a:solidFill
            $objWriter->startElement('a:solidFill');
            $this->writeColor($objWriter, $pBorder->getColor());
            $objWriter->endElement();
        }

        // Dash
        $objWriter->startElement('a:prstDash');
        $objWriter->writeAttribute('val', $pBorder->getDashStyle());
        $objWriter->endElement();

        // a:round
        $objWriter->writeElement('a:round', null);

        if ('both' === $lineEnd || 'head' === $lineEnd) {
            // a:headEnd
            $objWriter->startElement('a:headEnd');
            $objWriter->writeAttribute('type', 'triangle');
            $objWriter->endElement();
        }

        if ('both' === $lineEnd || 'tail' === $lineEnd) {
            // a:tailEnd
            $objWriter->startElement('a:tailEnd');
            $objWriter->writeAttribute('type', 'triangle');
            $objWriter->endElement();
        }

        $objWriter->endElement();
    }

    /**
     * @param XMLWriter $objWriter
     * @param Color $color
     * @param int|null $alpha
     */
    protected function writeColor(XMLWriter $objWriter, Color $color, $alpha = null)
    {
        if ($color instanceof SchemeColor) {
            $objWriter->startElement('a:schemeClr');
            $objWriter->writeAttribute('val', $color->getValue());
            $objWriter->endElement();
            return;
        }

        if (is_null($alpha)) {
            $alpha = $color->getAlpha();
        }
        $alpha = round(floatval($alpha)) * 1000;

        // a:srgbClr
        $objWriter->startElement('a:srgbClr');
        $objWriter->writeAttribute('val', $color->getRGB());

        // a:alpha
        $objWriter->startElement('a:alpha');
        //$objWriter->writeAttribute('val', $alpha . '%');
        $objWriter->writeAttribute('val', $alpha);
        $objWriter->endElement();

        $objWriter->endElement();
    }

    /**
     * Write Fill
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param  \PhpOffice\PhpPresentation\Style\Fill       $pFill     Fill style
     * @throws \Exception
     */
    protected function writeFill(XMLWriter $objWriter, $pFill)
    {
        if (! $pFill instanceof Fill) {
            return;
        }

        // Is it a fill?
        if ($pFill->getFillType() == Fill::FILL_NONE) {
            $objWriter->writeElement('a:noFill');
            return;
        }

        // Is it a solid fill?
        if ($pFill->getFillType() == Fill::FILL_SOLID) {
            $this->writeSolidFill($objWriter, $pFill);
            return;
        }

        if (Fill::FILL_BLIP == $pFill->getFillType()) {
            $rId = $this->getRelationId();
            $this->addRelation($rId, 'image', '../media/' . $pFill->getIndexedFilename());

            $objWriter->startElement('a:blipFill');
            $objWriter->writeAttribute('dpi', 0);
            $objWriter->writeAttribute('rotWithShape', 1);

            $objWriter->startElement('a:blip');
            $objWriter->writeAttribute('r:embed', $rId);

            $objWriter->startElement('a:extLst');
            $objWriter->startElement('a:ext');
            $objWriter->writeAttribute('uri', '{28A0092B-C50C-407E-A947-70E740481C1C}');

            $objWriter->startElement('a14:useLocalDpi');
            $objWriter->writeAttribute('xmlns:a14', 'http://schemas.microsoft.com/office/drawing/2010/main');
            $objWriter->writeAttribute('val', 0);
            $objWriter->endElement();

            $objWriter->endElement();
            $objWriter->endElement();
            $objWriter->endElement();

            $objWriter->writeElement('a:srcRect', null);
            $objWriter->startElement('a:stretch');
            $objWriter->writeElement('a:fillRect', null);
            $objWriter->endElement();

            $objWriter->endElement();

            $this->getZip()->addFromString('ppt/media/' . $pFill->getIndexedFilename(), file_get_contents($pFill->getImageSource()));
            return;
        }

        // Check if this is a pattern type or gradient type
        if ($pFill->getFillType() == Fill::FILL_GRADIENT_LINEAR || $pFill->getFillType() == Fill::FILL_GRADIENT_PATH) {
            // Gradient fill
            $this->writeGradientFill($objWriter, $pFill);
        } else {
            // Pattern fill
            $this->writePatternFill($objWriter, $pFill);
        }
    }

    /**
     * Write Solid Fill
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param  \PhpOffice\PhpPresentation\Style\Fill       $pFill     Fill style
     * @throws \Exception
     */
    protected function writeSolidFill(XMLWriter $objWriter, Fill $pFill)
    {
        // a:gradFill
        $objWriter->startElement('a:solidFill');
        $this->writeColor($objWriter, $pFill->getStartColor());
        $objWriter->endElement();
    }

    /**
     * Write Gradient Fill
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param  \PhpOffice\PhpPresentation\Style\Fill       $pFill     Fill style
     * @throws \Exception
     */
    protected function writeGradientFill(XMLWriter $objWriter, Fill $pFill)
    {
        // a:gradFill
        $objWriter->startElement('a:gradFill');

        // a:gsLst
        $objWriter->startElement('a:gsLst');
        // a:gs
        $objWriter->startElement('a:gs');
        $objWriter->writeAttribute('pos', '0%');
        $this->writeColor($objWriter, $pFill->getStartColor());
        $objWriter->endElement();

        // a:gs
        $objWriter->startElement('a:gs');
        $objWriter->writeAttribute('pos', '100%');
        $this->writeColor($objWriter, $pFill->getEndColor());
        $objWriter->endElement();

        $objWriter->endElement();

        // a:lin
        $objWriter->startElement('a:lin');
        $objWriter->writeAttribute('ang', CommonDrawing::degreesToAngle($pFill->getRotation()));
        $objWriter->writeAttribute('scaled', '0');
        $objWriter->endElement();

        $objWriter->endElement();
    }

    /**
     * Write Pattern Fill
     *
     * @param  \PhpOffice\Common\XMLWriter $objWriter XML Writer
     * @param  \PhpOffice\PhpPresentation\Style\Fill       $pFill     Fill style
     * @throws \Exception
     */
    protected function writePatternFill(XMLWriter $objWriter, Fill $pFill)
    {
        // a:pattFill
        $objWriter->startElement('a:pattFill');

        // fgClr
        $objWriter->startElement('a:fgClr');

        $this->writeColor($objWriter, $pFill->getStartColor());

        $objWriter->endElement();

        // bgClr
        $objWriter->startElement('a:bgClr');

        $this->writeColor($objWriter, $pFill->getEndColor());

        $objWriter->endElement();

        $objWriter->endElement();
    }

    /**
     * Write Outline
     * @param XMLWriter $objWriter
     * @param Outline $oOutline
     */
    protected function writeOutline(XMLWriter $objWriter, $oOutline)
    {
        if (!$oOutline instanceof Outline) {
            return;
        }
        // Width : pts
        $width = $oOutline->getWidth();
        // Width : pts => px
        $width = CommonDrawing::pointsToPixels($width);
        // Width : px => emu
        $width = CommonDrawing::pixelsToEmu($width);

        // a:ln
        $objWriter->startElement('a:ln');
        $objWriter->writeAttribute('w', $width);

        // Fill
        $this->writeFill($objWriter, $oOutline->getFill());

        // > a:ln
        $objWriter->endElement();
    }

    /**
     * Determine absolute zip path
     *
     * @param  string $path
     * @return string
     */
    protected function absoluteZipPath($path)
    {
        $path      = str_replace(array(
            '/',
            '\\'
        ), DIRECTORY_SEPARATOR, $path);
        $parts     = array_filter(explode(DIRECTORY_SEPARATOR, $path), 'strlen');
        $absolutes = array();
        foreach ($parts as $part) {
            if ('.' == $part) {
                continue;
            }
            if ('..' == $part) {
                array_pop($absolutes);
            } else {
                $absolutes[] = $part;
            }
        }

        return implode('/', $absolutes);
    }

    protected function writeParagraphProperty(XMLWriter $objWriter, $font)
    {
        if (0 !== $font->getSize()) {
            $objWriter->writeAttribute('sz', ($font->getSize() * 100));
        }
        if (Font::STATE_NOTSET !== $font->getBold()) {
            $objWriter->writeAttribute('b', Font::STATE_ON === $font->getBold() ? '1' : '0');
        }
        if (Font::STATE_NOTSET !== $font->getItalic()) {
            $objWriter->writeAttribute('i', Font::STATE_ON === $font->getItalic() ? '1' : '0');
        }
        if (Font::STATE_NOTSET !== $font->getStrikethrough()) {
            $objWriter->writeAttribute('strike', Font::STATE_ON === $font->getStrikethrough() ? 'sngStrike' : 'noStrike');
        }
        if (Font::STATE_NOTSET !== $font->getSuperScript()) {
            $objWriter->writeAttribute('baseline', Font::STATE_ON === $font->getSuperScript() ? '30000' : '0');
        }
        if (Font::STATE_NOTSET !== $font->getSubScript()) {
            $objWriter->writeAttribute('baseline', Font::STATE_ON === $font->getSubScript() ? '-25000' : '0');
        }
        if (Font::UNDERLINE_NOTSET !== $font->getUnderline()) {
            $objWriter->writeAttribute('u', Font::UNDERLINE_NONE === $font->getUnderline() ? Font::UNDERLINE_NONE : Font::UNDERLINE_SINGLE);
        }
        if (!is_null($font->getColor())) {
            $objWriter->startElement('a:solidFill');
            $this->writeColor($objWriter, $font->getColor());
            $objWriter->endElement();
        }
        if (!is_null($font->getHighlight())) {
            $objWriter->startElement('a:highlight');
            $this->writeColor($objWriter, $font->getHighlight());
            $objWriter->endElement();
        }
        if ('' !== $font->getName()) {
            $objWriter->startElement('a:latin');
            $objWriter->writeAttribute('typeface', $font->getName());
            $objWriter->endElement();

            $objWriter->startElement('a:ea');
            $objWriter->writeAttribute('typeface', $font->getName());
            $objWriter->endElement();

            $objWriter->startElement('a:cs');
            $objWriter->writeAttribute('typeface', $font->getName());
            $objWriter->endElement();
        }
    }
}
