<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\HTML_ProseMirror\Style;

use PhpOffice\PhpWord\Style\Font as FontStyle;

/**
 * Font style HTML writer
 *
 * @since 0.10.0
 */
class Font extends AbstractStyle
{
    /**
     * Write style
     *
     * @return string
     */
    public function write()
    {
        $style = $this->getStyle()->getFinalStyle();
        if (!$style instanceof FontStyle) {
            return null;
        }

        $font = $style->getName();
        $size = $style->getSize();
        $color = $style->getColor();
        $fgColor = $style->getFgColor();
        $bgColor = $style->getBgColor();
        $underline = !is_null($style->getUnderline()) && ($style->getUnderline() != FontStyle::UNDERLINE_NONE);
        $lineThrough = $style->isStrikethrough() || $style->isDoubleStrikethrough();

        $colorCodeToRgbValue = function ($color) {
            switch ($color) {
                case 'darkGray':
                    return '#a9a9a9';
                case 'lightGray':
                    return '#d3d3d3';
                case 'white':
                    return '#ffffff';
                case 'darkRed':
                    return '#8b0000';
                case 'red':
                    return '#ff0000';
                case 'darkYellow':
                    return '#8b8b00';
                case 'yellow':
                    return '#ffff00';
                case 'darkGreen':
                    return '#008b00';
                case 'green':
                    return '#00ff00';
                case 'darkCyan':
                    return '#008b8b';
                case 'cyan':
                    return '#00ffff';
                case 'darkBlue':
                    return '#00008b';
                case 'blue':
                    return '#0000ff';
                case 'darkMagenta':
                    return '#8b008b';
                case 'magenta':
                    return '#ff00ff';
                default:
                    return $color;
            }
        };

        $openingTags = '';
        $closingTags = '';
        if ($font !== null) {
            $openingTags .= "<span style=\"font-family:{$font};\">";
            $closingTags = '</span>' . $closingTags;
        }
        if ($size !== null) {
            $openingTags .= "<span style=\"font-size:{$size}pt;\">";
            $closingTags = '</span>' . $closingTags;
        }
        if ($color !== null) {
            $openingTags .= "<span style=\"color:#{$color};\">";
            $closingTags = '</span>' . $closingTags;
        }
        if ($bgColor != '') {
            $openingTags .= "<span style=\"background-color:#{$bgColor};\">";
            $closingTags = '</span>' . $closingTags;
        }
        if ($fgColor != '') {
            // fgColor comes from w:highlight, which is just like background color
            $openingTags .= "<span style=\"background-color:{$colorCodeToRgbValue($fgColor)};\">";
            $closingTags = '</span>' . $closingTags;
        }
        if ($style->isBold()) {
            $openingTags .= '<strong>';
            $closingTags = '</strong>' . $closingTags;
        }
        if ($style->isItalic()) {
            $openingTags .= '<em>';
            $closingTags = '</em>' . $closingTags;
        }
        if ($style->isSuperScript()) {
            $openingTags .= '<sup>';
            $closingTags = '</sup>' . $closingTags;
        }
        if ($style->isSubScript()) {
            $openingTags .= '<sub>';
            $closingTags = '</sub>' . $closingTags;
        }
        if ($underline) {
            $openingTags .= '<u>';
            $closingTags = '</u>' . $closingTags;
        }
        if ($lineThrough) {
            $openingTags .= '<s>';
            $closingTags = '</s>' . $closingTags;
        }
        if (!$openingTags) {
            $openingTags = '<span>';
        }
        if (!$closingTags) {
            $closingTags = '</span>';
        }
        return array('openingTags' => $openingTags, 'closingTags' => $closingTags);
    }

    public function writeCSS()
    {
        $style = $this->getStyle()->getFinalStyle();
        if (!$style instanceof FontStyle) {
            return null;
        }

        $font = $style->getName();
        $size = $style->getSize();
        $color = $style->getColor();
        $fgColor = $style->getFgColor();
        $bgColor = $style->getBgColor();
        $underline = !is_null($style->getUnderline()) && ($style->getUnderline() != FontStyle::UNDERLINE_NONE);
        $lineThrough = $style->isStrikethrough() || $style->isDoubleStrikethrough();

        $colorCodeToRgbValue = function ($color) {
            switch ($color) {
                case 'darkGray':
                    return '#a9a9a9';
                case 'lightGray':
                    return '#d3d3d3';
                case 'white':
                    return '#ffffff';
                case 'darkRed':
                    return '#8b0000';
                case 'red':
                    return '#ff0000';
                case 'darkYellow':
                    return '#8b8b00';
                case 'yellow':
                    return '#ffff00';
                case 'darkGreen':
                    return '#008b00';
                case 'green':
                    return '#00ff00';
                case 'darkCyan':
                    return '#008b8b';
                case 'cyan':
                    return '#00ffff';
                case 'darkBlue':
                    return '#00008b';
                case 'blue':
                    return '#0000ff';
                case 'darkMagenta':
                    return '#8b008b';
                case 'magenta':
                    return '#ff00ff';
                default:
                    return $color;
            }
        };

        $css = array();
        if ( !is_null($font) ) {
            $css['font-family'] = $font;
        }
        if ( !is_null($size) ) {
            $css['font-size'] = $size . 'pt';
        }
        if ( !is_null($color) ) {
            $css['color'] = '#' . $color;
        }
        if ( $bgColor != '' ) {
            $css['background-color'] = '#' . $bgColor;
        }
        if ( $fgColor != '' ) {
            // fgColor comes from w:highlight, which is just like background color
            $css['background-color'] = $colorCodeToRgbValue($fgColor);
        }
        if ( $style->isBold() ) {
            $css['font-weight'] = 'bold';
        }
        if ( $style->isItalic() ) {
            $css['font-style'] = 'italic';
        }
        if ( $style->isSuperScript() ) {
            $css['vertical-align'] = 'super';
        }
        if ( $style->isSubScript() ) {
            $css['vertical-align'] = 'sub';
        }
        // FIXME: underline & line-though cannot exist together in CSS
        if ( $underline ) {
            $css['text-decoration'] = 'underline';
        }
        if ( $lineThrough ) {
            $css['text-decoration'] = 'line-through';
        }
        return $this->assembleCss($css);
    }
}
