<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\Word2007\Element;

use PhpOffice\PhpWord\Element\AbstractElement as Element;
use PhpOffice\PhpWord\Element\TextBreak as TextBreakElement;
use PhpOffice\PhpWord\Style\Font as Font;
use PhpOffice\PhpWord\Shared\XMLWriter;
use PhpOffice\PhpWord\Writer\Word2007\Style\Paragraph as ParagraphStyleWriter;
use PhpOffice\PhpWord\Writer\Word2007\Style\Font as FontStyleWriter;

/**
 * ListItemRun element writer
 *
 * @since 0.10.0
 */
class ListItemRun extends AbstractElement
{
    /**
     * Write list item element.
     *
     * @return void
     */
    public function write()
    {
        $xmlWriter = $this->getXmlWriter();
        $element = $this->getElement();
        if (!$element instanceof \PhpOffice\PhpWord\Element\ListItemRun) {
            return;
        }
        // synoListItem marker style
        $fontWriter = null;
        if ( is_array($element->getParagraphStyle()->getSynoListItem()) ) {
            $fontStyle = new Font();
            $fontStyle->setStyleByArray($element->getParagraphStyle()->getSynoListItem());
            $fontWriter = new FontStyleWriter($xmlWriter, $fontStyle);
        }

        // Copy from ./Container.php

        $container = $element;
        $containerClass = 'ListItemRun'; // substr(get_class($container), strrpos(get_class($container), '\\') + 1);
        $withoutP = true; // in_array($containerClass, array('TextRun', 'Footnote', 'Endnote', 'ListItemRun', 'TitleRun', 'Link')) ? true : false;

        // Loop through elements
        $elements = $container->getElements();
        $elementClass = '';
        $firstElement = true;
        foreach ($elements as $element) {
            $elementClass = get_class($element);
            $writeNumbering = (substr($elementClass, strrpos($elementClass, '\\') + 1) !== 'Table');
            if ($writeNumbering) {
                $xmlWriter->startElement('w:p');

                $xmlWriter->startElement('w:pPr');
                $paragraphStyle = $container->getParagraphStyle();
                $styleWriter = new ParagraphStyleWriter($xmlWriter, $paragraphStyle);
                $styleWriter->setIsInline(true);
                $styleWriter->setWithoutPPR(true);
                $styleWriter->write();

                $elemPragraphStyle = $element->getParagraphStyle();
                $styleWriter = new ParagraphStyleWriter($xmlWriter, $elemPragraphStyle);
                $styleWriter->setWithoutPPR(true);
                $styleWriter->write();

                $indent;
                if ( $firstElement ) {
                    $firstElement = false;
                    $indent = intval($paragraphStyle->getIndent()) + 810 * (1 + $container->getDepth());
                    $paragraphStyle->setIndent($indent);
                    $xmlWriter->startElement('w:numPr');
                    $xmlWriter->startElement('w:ilvl');
                    $xmlWriter->writeAttribute('w:val', $container->getDepth());
                    $xmlWriter->endElement(); // w:ilvl
                    $xmlWriter->startElement('w:numId');
                    $xmlWriter->writeAttribute('w:val', $container->getStyle()->getNumId());
                    $xmlWriter->endElement(); // w:numId
                    $xmlWriter->endElement(); // w:numPr
                }

                if ( !is_null($fontWriter) ) {
                    $fontWriter->write();
                }

                $xmlWriter->endElement(); // w:pPr
            } else {
                $element->getStyle()->setIndent(810 * (1 + $container->getDepth()));
            }

            $elementClass = $this->writeElement($xmlWriter, $element, $withoutP);

            if ($writeNumbering) {
                $xmlWriter->endElement(); // w:p
            }
        }

        // Special case for Cell: They have to contain a w:p element at the end.
        // The $elementClass contains the last element name. If it's empty string
        // or Table, the last element is not w:p
        $writeLastTextBreak = (($containerClass == 'Cell') && (($elementClass == '') || ($elementClass == 'Table')));
        if ($writeLastTextBreak) {
            $writerClass = 'PhpOffice\\PhpWord\\Writer\\Word2007\\Element\\TextBreak'; // $this->namespace . '\\TextBreak';
            /** @var \PhpOffice\PhpWord\Writer\Word2007\Element\AbstractElement $writer Type hint */
            $writer = new $writerClass($xmlWriter, new TextBreakElement(), $withoutP);
            $writer->write();
        }
    }

    /**
     * Write individual element
     *
     * @param \PhpOffice\PhpWord\Shared\XMLWriter $xmlWriter
     * @param \PhpOffice\PhpWord\Element\AbstractElement $element
     * @param bool $withoutP
     * @return string
     */
    private function writeElement(XMLWriter $xmlWriter, Element $element, $withoutP)
    {
        $elementClass = substr(get_class($element), strrpos(get_class($element), '\\') + 1);
        $writerClass = 'PhpOffice\\PhpWord\\Writer\\Word2007\\Element\\' . $elementClass; // $this->namespace . '\\' . $elementClass;

        if (class_exists($writerClass)) {
            /** @var \PhpOffice\PhpWord\Writer\Word2007\Element\AbstractElement $writer Type hint */
            $writer = new $writerClass($xmlWriter, $element, $withoutP);
            $writer->write();
        }

        return $elementClass;
    }
}
